/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.entity.adapter;

import java.util.HashMap;
import java.util.Map;

import mir.entity.Entity;
import mir.storage.Database;

public class EntityAdapterModel {
  private Map entityAdapterMappings;

  public EntityAdapterModel() {
    entityAdapterMappings = new HashMap();
  }

  public EntityAdapter makeEntityAdapter( String aName, Entity anEntity ) {
    Mapping mapping = getMappingForName( aName );

    return mapping.getDefinition().makeEntityAdapter( anEntity, this );
  }

  public EntityAdapter createNewEntity( String aName ) throws EntityAdapterExc {
    try {
      Mapping mapping = getMappingForName(aName);
      Entity entity = mapping.database.createNewEntity();

      return mapping.getDefinition().makeEntityAdapter(entity, this);
    }
    catch (Throwable t) {
      throw new EntityAdapterFailure(t);
    }
  }

  public void addMapping(String aName, Database aDatabase, EntityAdapterDefinition aDefinition ) {
    entityAdapterMappings.put(aName, new Mapping(aDatabase, aDefinition));
  }

  public Mapping getMappingForName( String aName ) {
    if (!entityAdapterMappings.containsKey(aName))
      throw new EntityAdapterFailure("'" + aName + "' not present in model", null);

    return (Mapping) entityAdapterMappings.get( aName );
  }

  public class Mapping {
    private Database database;
    private EntityAdapterDefinition definition;

    public Mapping(Database aDatabase, EntityAdapterDefinition aDefinition ) {
      database = aDatabase;
      definition = aDefinition;
    }

    public Database getDatabase() {
      return database;
    }

    public EntityAdapterDefinition getDefinition() {
      return definition;
    }
  }
}


