/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.servlet;

import mir.entity.adapter.EntityAdapterEngine;
import mir.servlet.AdminServletModule;
import mir.servlet.ServletModuleExc;
import mir.servlet.ServletModuleFailure;
import mir.util.HTTPRequestParser;
import mir.util.JDBCStringRoutines;
import mircoders.module.ModuleContent;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/*
 *  ServletModuleHidden - output of so called "censored" articles
 *  @author mh
 *  @version $Id
 */

public class ServletModuleHidden extends AdminServletModule {
  public ServletModuleHidden() {
    definition = "content";
    mainModule = new ModuleContent();
  }


  public void list(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);
    Map responseData = ServletHelper.makeGenerationData(aRequest, aResponse, new Locale[] { getLocale(aRequest), getFallbackLocale(aRequest)});

    String queryYear = requestParser.getParameter("year");
    String queryMonth = requestParser.getParameter("month");

    try {
      if ((queryYear !=null && !queryYear.equals("")) &&
          (queryMonth !=null && !queryMonth.equals(""))) {
        String whereClause = "is_published=false AND webdb_create LIKE "+
            "'"+JDBCStringRoutines.escapeStringLiteral(queryYear)+"-"+
               JDBCStringRoutines.escapeStringLiteral(queryMonth)+"%'";


        List articleList = EntityAdapterEngine.retrieveAdapterList(getModel(), definition, whereClause, "webdb_create", -1, 0);

        responseData.put("year", queryYear);
        responseData.put("month", queryMonth);
        responseData.put("articles", articleList);
      }

      ServletHelper.generateResponse(aResponse.getWriter(), responseData, listGenerator);
    }
    catch (Throwable e) {
      throw new ServletModuleFailure(e);
    }
  }
}
